<?php

/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2019 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit();
}

require_once dirname(__FILE__) . '/classes/BK.php';

class BifitKassa extends Module
{
    private $_html = '';
    private $_postErrors = array();

    public $tab_array = array(
        array(
            'class_name' => 'BifitFiscalCheck',
            'name' => 'БИФИТ Касса'
        )
    );

    public function __construct()
    {
        $this->name = 'bifitkassa';
        $this->tab = 'payments_gateways';
        $this->version = '1.2.4';
        $this->author = 'БИФИТ';
        $this->need_instance = 0;
        $this->module_key = '';

        /**
         * Module is compliant with bootstrap
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('БИФИТ Касса');
        $this->description = $this->l(
            'Модуль для фискализации платежей по 54-ФЗ от БИФИТ Касса.'
        );
        $this->ps_versions_compliancy = array(
            'min' => '1.6',
            'max' => '1.7.99.99'
        );
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        if (!parent::install()) {
            return false;
        }

        $sql = array();
        include dirname(__FILE__) . '/init/install_sql.php';
        foreach ($sql as $s) {
            if (!Db::getInstance()->Execute($s)) {
                return false;
            }
        }

        foreach ($this->tab_array as $tab) {
            $new_tab = new Tab();
            $new_tab->class_name = $tab['class_name'];
            $new_tab->id_parent = (int) Tab::getIdFromClassName('AdminAdmin');
            $new_tab->module = $this->name;

            $languages = Language::getLanguages();
            foreach ($languages as $language) {
                $new_tab->name[$language['id_lang']] = $tab['name'];
            }

            $new_tab->add();
        }

        $rand = rand(1111, 999999999);
        $rand_hex = dechex($rand);
        $log_file_name = md5($rand_hex);

        if (!Hook::getIdByName('scapiInit')) {
            $hook = new Hook();
            $hook->name = 'scapiInit';
            $hook->title = $this->displayName;
            $hook->description = $this->description;
            $hook->add();
        }

        Configuration::updateValue('bifitkassa_test', 1);
        Configuration::updateValue(
            'bifitkassa_token',
            'P5cKbUUD9uSSrSlGdzspLblvBnD0GzTAE0cLmAPSEMxJ79DtLE'
        );
        Configuration::updateValue('bifitkassa_cashier_name', 'Иванов И.И.');
        Configuration::updateValue('bifitkassa_cashier_inn', '123456789012');
        Configuration::updateValue(
            'bifitkassa_payment_address',
            'https://site.ru'
        );
        Configuration::updateValue('bifitkassa_address_type', 'email');
        Configuration::updateValue('bifitkassa_fio', 0);
        Configuration::updateValue('bifitkassa_calculation_method', 'FULL_PAY');
        Configuration::updateValue('bifitkassa_payment_type', 'CARD');
        Configuration::updateValue('bifitkassa_vat_mode', 1);
        Configuration::updateValue('bifitkassa_vat', 'WITHOUT_VAT');
        Configuration::updateValue('bifitkassa_use_type_1', 0);
        Configuration::updateValue(
            'bifitkassa_tax_system_1',
            'SIMPLIFIED_WITH_EXPENSE'
        );
        Configuration::updateValue('bifitkassa_pm_ids_1', []);
        Configuration::updateValue('bifitkassa_pm_statuses_1', []);
        Configuration::updateValue('bifitkassa_use_type_2', 0);
        Configuration::updateValue(
            'bifitkassa_tax_system_2',
            'SIMPLIFIED_WITH_EXPENSE'
        );
        Configuration::updateValue('bifitkassa_pm_ids_2', []);
        Configuration::updateValue('bifitkassa_pm_statuses_2', []);
        Configuration::updateValue('bifitkassa_use_type_3', 0);
        Configuration::updateValue(
            'bifitkassa_tax_system_3',
            'SIMPLIFIED_WITH_EXPENSE'
        );
        Configuration::updateValue('bifitkassa_pm_ids_3', []);
        Configuration::updateValue('bifitkassa_pm_statuses_3', []);
        Configuration::updateValue('bifitkassa_use_type_4', 0);
        Configuration::updateValue(
            'bifitkassa_tax_system_4',
            'SIMPLIFIED_WITH_EXPENSE'
        );
        Configuration::updateValue('bifitkassa_pm_ids_4', []);
        Configuration::updateValue('bifitkassa_pm_statuses_4', []);

        if (!$this->registerHook('actionOrderStatusUpdate')) {
            return false;
        }

        return true;
    }

    public function uninstall()
    {
        if ($id_hook = Hook::getIdByName('scapiInit')) {
            $hook = new Hook($id_hook);
            $hook->delete();
        }

        $sql = array();
        include dirname(__FILE__) . '/init/uninstall_sql.php';
        foreach ($sql as $s) {
            if (!Db::getInstance()->Execute($s)) {
                return false;
            }
        }

        $idTabs = array();
        $idTabs[] = Tab::getIdFromClassName('Bifit FiscalCheck');
        foreach ($idTabs as $idTab) {
            if ($idTab) {
                $tab = new Tab($idTab);
                $tab->delete();
            }
        }

        $log_file_name = Configuration::get('log_file_name');
        $file_name =
            dirname(__DIR__) . '/bifitkassa/' . $log_file_name . '.txt';
        if (is_readable($file_name)) {
            $result = unlink($file_name);
        }

        return parent::uninstall() &&
            Configuration::deleteByName('bifitkassa_test') &&
            Configuration::deleteByName('bifitkassa_token') &&
            Configuration::deleteByName('bifitkassa_cashier_name') &&
            Configuration::deleteByName('bifitkassa_cashier_inn') &&
            Configuration::deleteByName('bifitkassa_payment_address') &&
            Configuration::deleteByName('bifitkassa_address_type') &&
            Configuration::deleteByName('bifitkassa_fio') &&
            Configuration::deleteByName('bifitkassa_calculation_method') &&
            Configuration::deleteByName('bifitkassa_payment_type') &&
            Configuration::deleteByName('bifitkassa_vat_mode') &&
            Configuration::deleteByName('bifitkassa_vat') &&
            Configuration::deleteByName('bifitkassa_vat_mode') &&
            Configuration::deleteByName('bifitkassa_use_type_1') &&
            Configuration::deleteByName('bifitkassa_tax_system_1') &&
            Configuration::deleteByName('bifitkassa_pm_ids_1') &&
            Configuration::deleteByName('bifitkassa_pm_statuses_1') &&
            Configuration::deleteByName('bifitkassa_use_type_2') &&
            Configuration::deleteByName('bifitkassa_tax_system_2') &&
            Configuration::deleteByName('bifitkassa_pm_ids_2') &&
            Configuration::deleteByName('bifitkassa_pm_statuses_2') &&
            Configuration::deleteByName('bifitkassa_use_type_3') &&
            Configuration::deleteByName('bifitkassa_tax_system_3') &&
            Configuration::deleteByName('bifitkassa_pm_ids_3') &&
            Configuration::deleteByName('bifitkassa_pm_statuses_3') &&
            Configuration::deleteByName('bifitkassa_use_type_4') &&
            Configuration::deleteByName('bifitkassa_tax_system_4') &&
            Configuration::deleteByName('bifitkassa_pm_ids_4') &&
            Configuration::deleteByName('bifitkassa_pm_statuses_4');
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        if (Tools::isSubmit('submitscapi')) {
            $this->_postValidation();
            if (!sizeof($this->_postErrors)) {
                $this->_postProcess();
            } else {
                foreach ($this->_postErrors as $err) {
                    $this->_html .= $this->displayError($err);
                }
            }
        }
        $this->_displayForm();

        return $this->_html;
    }

    private function initToolbar()
    {
        $this->toolbar_btn['save'] = array(
            'href' => '#',
            'desc' => $this->l('Сохранено')
        );

        return $this->toolbar_btn;
    }

    protected function _displayForm()
    {
        $this->_html .=
            '<h2><span style="color:#000000; font-weight:700;">БИФИТ <span style="color:#67AEF7;">Касса</span></span></h2>';

        $options = array(
            array(
                'id_option' => 1,
                'name' => 'без НДС'
            ),
            array(
                'id_option' => 2,
                'name' => 'НДС 0%'
            ),
            array(
                'id_option' => 10,
                'name' => 'НДС 10%'
            ),
            array(
                'id_option' => 18,
                'name' => 'НДС 18%'
            ),
            array(
                'id_option' => 110,
                'name' => 'НДС 10/110'
            ),
            array(
                'id_option' => 118,
                'name' => 'НДС 18/118'
            )
        );

        $op_status = array();

        $sql = 'SELECT * FROM `' . _DB_PREFIX_ . 'order_state_lang`';
        if ($results = Db::getInstance()->ExecuteS($sql)) {
            foreach ($results as $row) {
                $name_id = $row['name'];
                $add_arr = array(
                    'id_option' => $row['id_order_state'],
                    'name' => $name_id
                );
                array_push($op_status, $add_arr);
            }
        }

        $payement_metods = array();
        $modules_list = Module::getPaymentModules();
        foreach ($modules_list as $module) {
            $module_obj = Module::getInstanceById($module['id_module']);
            $name_id = $module_obj->displayName;
            $add_arr = array(
                'id_option' => $module['id_module'],
                'name' => $name_id
            );
            array_push($payement_metods, $add_arr);
        }

        $this->_html .= '';

        $this->_display = 'index';

        $this->fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Настройки'),
                'icon' => 'icon-cogs'
            ),
            'input' => array(
                array(
                    'type' => 'select',
                    'label' => $this->l('Тестовый режим'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_test',
                    //'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 1,
                                'name' => 'Да'
                            ),
                            array(
                                'id_option' => 0,
                                'name' => 'нет'
                            )
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Токен доступа'),
                    'suffix' => $this->l('Токен доступа'),
                    'name' => 'bifitkassa_token',
                    'size' => 60,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('ФИО кассира'),
                    'suffix' => $this->l('ФИО кассира'),
                    'name' => 'bifitkassa_cashier_name',
                    'size' => 60,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('ИНН кассира'),
                    'suffix' => $this->l('ИНН кассира'),
                    'name' => 'bifitkassa_cashier_inn',
                    'size' => 60,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Место расчётов'),
                    'suffix' => $this->l('Место расчётов'),
                    'name' => 'bifitkassa_payment_address',
                    'size' => 60,
                    'required' => true
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Адрес клиента'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_address_type',
                    'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 'email',
                                'name' => 'Email'
                            ),
                            array(
                                'id_option' => 'phone',
                                'name' => 'Телефон'
                            )
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('ФИО клиента'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_fio',
                    //'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => '0',
                                'name' => 'Нет'
                            ),
                            array(
                                'id_option' => '1',
                                'name' => 'Да'
                            )
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Признак способа расчёта'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_calculation_method',
                    'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 'PREPAY_FULL',
                                'name' =>
                                    'Полная предварительная оплата до момента передачи предмета расчёта'
                            ),
                            array(
                                'id_option' => 'PREPAY_PARTIAL',
                                'name' =>
                                    'Частичная предварительная оплата до момента передачи предмета расчёта'
                            ),
                            array(
                                'id_option' => 'AVANS',
                                'name' => 'Аванс'
                            ),
                            array(
                                'id_option' => 'FULL_PAY',
                                'name' => 'Полная оплата, в том числе с учётом аванса (предварительной оплаты) в
							момент передачи предмета расчёта'
                            ),
                            array(
                                'id_option' => 'PARTIAL_SETTLEMENT_AND_CREDIT',
                                'name' => 'Частичная оплата предмета расчёта в момент его передачи с последующей
							оплатой в кредит'
                            ),
                            array(
                                'id_option' => 'TRANSFER_ON_CREDIT',
                                'name' => 'Передача предмета расчёта без его оплаты в момент его передачи с последующей
							оплатой в кредит'
                            ),
                            array(
                                'id_option' => 'CREDIT_PAYMENT',
                                'name' => 'Оплата предмета расчёта после его передачи с оплатой в кредит (оплата
							кредита)'
                            )
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Тип оплаты'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_payment_type',
                    'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 'CASH',
                                'name' => 'Сумма оплаты наличными'
                            ),
                            array(
                                'id_option' => 'CARD',
                                'name' => 'Сумма оплата безналичными'
                            ),
                            array(
                                'id_option' => 'PREPAY',
                                'name' =>
                                    'Сумма оплаты предоплатой (зачётом аванса)'
                            ),
                            array(
                                'id_option' => 'POSTPAY',
                                'name' => 'Сумма оплаты постоплатой (в кредит)'
                            ),
                            array(
                                'id_option' => 'OTHER',
                                'name' =>
                                    'Сумма оплаты встречным предоставлением'
                            )
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Единый НДС'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_vat_mode',
                    'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => '1',
                                'name' => 'Да'
                            ),
                            array(
                                'id_option' => '0',
                                'name' => 'Нет'
                            )
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Ставка НДС'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_vat',
                    'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 'WITHOUT_VAT',
                                'name' => 'Без НДС'
                            ),
                            array(
                                'id_option' => 'VAT_0',
                                'name' => 'НДС 0%'
                            ),
                            array(
                                'id_option' => 'VAT_10',
                                'name' => 'НДС 10%'
                            ),
                            array(
                                'id_option' => 'VAT_20',
                                'name' => 'НДС 20%'
                            ),
                            array(
                                'id_option' => 'VAT_110',
                                'name' => 'НДС 10/110'
                            ),
                            array(
                                'id_option' => 'VAT_120',
                                'name' => 'НДС 20/120'
                            )
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'label',
                    'label' => '<h2>' . $this->l('Приход') . '</h2>'
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Использовать'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_use_type_1',
                    //'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => '1',
                                'name' => 'Да'
                            ),
                            array(
                                'id_option' => '0',
                                'name' => 'Нет'
                            )
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Система налогообложения'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_tax_system_1',
                    //'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 'COMMON',
                                'name' => 'ОСН'
                            ),
                            array(
                                'id_option' => 'SIMPLIFIED',
                                'name' => 'УСН доход'
                            ),
                            array(
                                'id_option' => 'SIMPLIFIED_WITH_EXPENSE',
                                'name' => 'УСН доход-расход'
                            ),
                            array(
                                'id_option' => 'ENVD',
                                'name' => 'ЕНВД'
                            ),
                            array(
                                'id_option' => 'COMMON_AGRICULTURAL',
                                'name' => 'ЕСХН'
                            ),
                            array(
                                'id_option' => 'PATENT',
                                'name' => 'ПАТЕНТ'
                            )
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Способ(ы) оплаты'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_pm_ids_1',
                    'multiple' => true,
                    //'required' => true,
                    'options' => array(
                        'query' => $payement_metods,
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Статус заказа'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_pm_statuses_1',
                    'multiple' => true,
                    'options' => array(
                        'query' => $op_status,
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'label',
                    'label' => '<h2>' . $this->l('Возврат прихода') . '</h2>'
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Использовать'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_use_type_2',
                    //'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => '1',
                                'name' => 'Да'
                            ),
                            array(
                                'id_option' => '0',
                                'name' => 'Нет'
                            )
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Система налогообложения'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_tax_system_2',
                    //'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 'COMMON',
                                'name' => 'ОСН'
                            ),
                            array(
                                'id_option' => 'SIMPLIFIED',
                                'name' => 'УСН доход'
                            ),
                            array(
                                'id_option' => 'SIMPLIFIED_WITH_EXPENSE',
                                'name' => 'УСН доход-расход'
                            ),
                            array(
                                'id_option' => 'ENVD',
                                'name' => 'ЕНВД'
                            ),
                            array(
                                'id_option' => 'COMMON_AGRICULTURAL',
                                'name' => 'ЕСХН'
                            ),
                            array(
                                'id_option' => 'PATENT',
                                'name' => 'ПАТЕНТ'
                            )
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Способ(ы) оплаты'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_pm_ids_2',
                    'multiple' => true,
                    //'required' => true,
                    'options' => array(
                        'query' => $payement_metods,
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Статус заказа'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_pm_statuses_2',
                    'multiple' => true,
                    'options' => array(
                        'query' => $op_status,
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'label',
                    'label' => '<h2>' . $this->l('Расход') . '</h2>'
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Использовать'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_use_type_3',
                    //'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => '1',
                                'name' => 'Да'
                            ),
                            array(
                                'id_option' => '0',
                                'name' => 'Нет'
                            )
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Система налогообложения'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_tax_system_3',
                    //'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 'COMMON',
                                'name' => 'ОСН'
                            ),
                            array(
                                'id_option' => 'SIMPLIFIED',
                                'name' => 'УСН доход'
                            ),
                            array(
                                'id_option' => 'SIMPLIFIED_WITH_EXPENSE',
                                'name' => 'УСН доход-расход'
                            ),
                            array(
                                'id_option' => 'ENVD',
                                'name' => 'ЕНВД'
                            ),
                            array(
                                'id_option' => 'COMMON_AGRICULTURAL',
                                'name' => 'ЕСХН'
                            ),
                            array(
                                'id_option' => 'PATENT',
                                'name' => 'ПАТЕНТ'
                            )
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Способ(ы) оплаты'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_pm_ids_3',
                    'multiple' => true,
                    //'required' => true,
                    'options' => array(
                        'query' => $payement_metods,
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Статус заказа'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_pm_statuses_3',
                    'multiple' => true,
                    'options' => array(
                        'query' => $op_status,
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'label',
                    'label' => '<h2>' . $this->l('Возврат расхода') . '</h2>'
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Использовать'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_use_type_4',
                    //'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => '1',
                                'name' => 'Да'
                            ),
                            array(
                                'id_option' => '0',
                                'name' => 'Нет'
                            )
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Система налогообложения'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_tax_system_4',
                    //'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 'COMMON',
                                'name' => 'ОСН'
                            ),
                            array(
                                'id_option' => 'SIMPLIFIED',
                                'name' => 'УСН доход'
                            ),
                            array(
                                'id_option' => 'SIMPLIFIED_WITH_EXPENSE',
                                'name' => 'УСН доход-расход'
                            ),
                            array(
                                'id_option' => 'ENVD',
                                'name' => 'ЕНВД'
                            ),
                            array(
                                'id_option' => 'COMMON_AGRICULTURAL',
                                'name' => 'ЕСХН'
                            ),
                            array(
                                'id_option' => 'PATENT',
                                'name' => 'ПАТЕНТ'
                            )
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Способ(ы) оплаты'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_pm_ids_4',
                    'multiple' => true,
                    'options' => array(
                        'query' => $payement_metods,
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Статус заказа'),
                    'desc' => $this->l(''),
                    'name' => 'bifitkassa_pm_statuses_4',
                    'multiple' => true,
                    'options' => array(
                        'query' => $op_status,
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                )
            ),
            'submit' => array(
                'name' => 'submitscapi',
                'title' => $this->l('Save'),
                'class' => 'btn btn-success pull-right'
            )
        );
        $this->fields_value['bifitkassa_test'] = Configuration::get(
            'bifitkassa_test'
        );
        $this->fields_value['bifitkassa_token'] = Configuration::get(
            'bifitkassa_token'
        );
        $this->fields_value['bifitkassa_cashier_name'] = Configuration::get(
            'bifitkassa_cashier_name'
        );
        $this->fields_value['bifitkassa_cashier_inn'] = Configuration::get(
            'bifitkassa_cashier_inn'
        );
        $this->fields_value['bifitkassa_payment_address'] = Configuration::get(
            'bifitkassa_payment_address'
        );
        $this->fields_value['bifitkassa_address_type'] = Configuration::get(
            'bifitkassa_address_type'
        );
        $this->fields_value['bifitkassa_fio'] = Configuration::get(
            'bifitkassa_fio'
        );
        $this->fields_value[
            'bifitkassa_calculation_method'
        ] = Configuration::get('bifitkassa_calculation_method');
        $this->fields_value['bifitkassa_payment_type'] = Configuration::get(
            'bifitkassa_payment_type'
        );
        $this->fields_value['bifitkassa_vat_mode'] = Configuration::get(
            'bifitkassa_vat_mode'
        );
        $this->fields_value['bifitkassa_vat'] = Configuration::get(
            'bifitkassa_vat'
        );
        $this->fields_value['bifitkassa_vat_mode'] = Configuration::get(
            'bifitkassa_vat_mode'
        );
        $this->fields_value['bifitkassa_use_type_1'] = Configuration::get(
            'bifitkassa_use_type_1'
        );
        $this->fields_value['bifitkassa_tax_system_1'] = Configuration::get(
            'bifitkassa_tax_system_1'
        );
        $this->fields_value['bifitkassa_pm_ids_1[]'] = json_decode(
            Configuration::get('bifitkassa_pm_ids_1'),
            true
        );
        $this->fields_value['bifitkassa_pm_statuses_1[]'] = json_decode(
            Configuration::get('bifitkassa_pm_statuses_1'),
            true
        );
        $this->fields_value['bifitkassa_use_type_2'] = Configuration::get(
            'bifitkassa_use_type_2'
        );
        $this->fields_value['bifitkassa_tax_system_2'] = Configuration::get(
            'bifitkassa_tax_system_2'
        );
        $this->fields_value['bifitkassa_pm_ids_2[]'] = json_decode(
            Configuration::get('bifitkassa_pm_ids_2'),
            true
        );
        $this->fields_value['bifitkassa_pm_statuses_2[]'] = json_decode(
            Configuration::get('bifitkassa_pm_statuses_2'),
            true
        );
        $this->fields_value['bifitkassa_use_type_3'] = Configuration::get(
            'bifitkassa_use_type_3'
        );
        $this->fields_value['bifitkassa_tax_system_3'] = Configuration::get(
            'bifitkassa_tax_system_3'
        );
        $this->fields_value['bifitkassa_pm_ids_3[]'] = json_decode(
            Configuration::get('bifitkassa_pm_ids_3'),
            true
        );
        $this->fields_value['bifitkassa_pm_statuses_3[]'] = json_decode(
            Configuration::get('bifitkassa_pm_statuses_3'),
            true
        );
        $this->fields_value['bifitkassa_use_type_4'] = Configuration::get(
            'bifitkassa_use_type_4'
        );
        $this->fields_value['bifitkassa_tax_system_4'] = Configuration::get(
            'bifitkassa_tax_system_4'
        );
        $this->fields_value['bifitkassa_pm_ids_4[]'] = json_decode(
            Configuration::get('bifitkassa_pm_ids_4'),
            true
        );
        $this->fields_value['bifitkassa_pm_statuses_4[]'] = json_decode(
            Configuration::get('bifitkassa_pm_statuses_4'),
            true
        );

        $helper = $this->initForm();
        $helper->submit_action = '';

        $helper->title = $this->displayName;

        $helper->fields_value = $this->fields_value;
        $this->_html .= $helper->generateForm($this->fields_form);

        return;
    }

    private function initForm()
    {
        $helper = new HelperForm();

        $helper->module = $this;
        $helper->name_controller = 'bifitkassa';
        $helper->identifier = $this->identifier;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex =
            AdminController::$currentIndex . '&configure=' . $this->name;
        $helper->toolbar_scroll = true;
        $helper->tpl_vars['version'] = $this->version;
        $helper->tpl_vars['author'] = $this->author;
        $helper->tpl_vars['this_path'] = $this->_path;
        $helper->toolbar_btn = $this->initToolbar();

        return $helper;
    }

    private function _postValidation()
    {
        if (
            Tools::getValue('api_app') &&
            !Validate::isInt(Tools::getValue('api_app'))
        ) {
            $this->_postErrors[] = $this->l('Invalid') . ' ' . $this->l('app');
        }
    }

    private function _postProcess()
    {
        Configuration::updateValue(
            'bifitkassa_test',
            Tools::getValue('bifitkassa_test')
        );
        Configuration::updateValue(
            'bifitkassa_token',
            Tools::getValue('bifitkassa_token')
        );
        Configuration::updateValue(
            'bifitkassa_cashier_name',
            Tools::getValue('bifitkassa_cashier_name')
        );
        Configuration::updateValue(
            'bifitkassa_cashier_inn',
            Tools::getValue('bifitkassa_cashier_inn')
        );
        Configuration::updateValue(
            'bifitkassa_payment_address',
            Tools::getValue('bifitkassa_payment_address')
        );
        Configuration::updateValue(
            'bifitkassa_address_type',
            Tools::getValue('bifitkassa_address_type')
        );
        Configuration::updateValue(
            'bifitkassa_fio',
            Tools::getValue('bifitkassa_fio')
        );
        Configuration::updateValue(
            'bifitkassa_calculation_method',
            Tools::getValue('bifitkassa_calculation_method')
        );
        Configuration::updateValue(
            'bifitkassa_payment_type',
            Tools::getValue('bifitkassa_payment_type')
        );
        Configuration::updateValue(
            'bifitkassa_vat_mode',
            Tools::getValue('bifitkassa_vat_mode')
        );
        Configuration::updateValue(
            'bifitkassa_vat',
            Tools::getValue('bifitkassa_vat')
        );
        Configuration::updateValue(
            'bifitkassa_vat_mode',
            Tools::getValue('bifitkassa_vat_mode')
        );
        Configuration::updateValue(
            'bifitkassa_use_type_1',
            Tools::getValue('bifitkassa_use_type_1')
        );
        Configuration::updateValue(
            'bifitkassa_tax_system_1',
            Tools::getValue('bifitkassa_tax_system_1')
        );
        Configuration::updateValue(
            'bifitkassa_pm_ids_1',
            json_encode(Tools::getValue('bifitkassa_pm_ids_1'))
        );
        Configuration::updateValue(
            'bifitkassa_pm_statuses_1',
            json_encode(Tools::getValue('bifitkassa_pm_statuses_1'))
        );
        Configuration::updateValue(
            'bifitkassa_use_type_2',
            Tools::getValue('bifitkassa_use_type_2')
        );
        Configuration::updateValue(
            'bifitkassa_tax_system_2',
            Tools::getValue('bifitkassa_tax_system_2')
        );
        Configuration::updateValue(
            'bifitkassa_pm_ids_2',
            json_encode(Tools::getValue('bifitkassa_pm_ids_2'))
        );
        Configuration::updateValue(
            'bifitkassa_pm_statuses_2',
            json_encode(Tools::getValue('bifitkassa_pm_statuses_2'))
        );
        Configuration::updateValue(
            'bifitkassa_use_type_3',
            Tools::getValue('bifitkassa_use_type_3')
        );
        Configuration::updateValue(
            'bifitkassa_tax_system_3',
            Tools::getValue('bifitkassa_tax_system_3')
        );
        Configuration::updateValue(
            'bifitkassa_pm_ids_3',
            json_encode(Tools::getValue('bifitkassa_pm_ids_3'))
        );
        Configuration::updateValue(
            'bifitkassa_pm_statuses_3',
            json_encode(Tools::getValue('bifitkassa_pm_statuses_3'))
        );
        Configuration::updateValue(
            'bifitkassa_use_type_4',
            Tools::getValue('bifitkassa_use_type_4')
        );
        Configuration::updateValue(
            'bifitkassa_tax_system_4',
            Tools::getValue('bifitkassa_tax_system_4')
        );
        Configuration::updateValue(
            'bifitkassa_pm_ids_4',
            json_encode(Tools::getValue('bifitkassa_pm_ids_4'))
        );
        Configuration::updateValue(
            'bifitkassa_pm_statuses_4',
            json_encode(Tools::getValue('bifitkassa_pm_statuses_4'))
        );

        $this->_html .= $this->displayConfirmation(
            $this->l('Настройки обновлены.')
        );
    }

    public static function __createGUID($order)
    {
        // Create GUID (Globally Unique Identifier)
        $guid = '';
        $namespace = rand(11111, 99999);
        $uid = uniqid('', true);
        $data = $namespace;
        $data .= $_SERVER['REQUEST_TIME'];
        $data .= $_SERVER['HTTP_USER_AGENT'];
        $data .= $_SERVER['REMOTE_ADDR'];
        $data .= $_SERVER['REMOTE_PORT'];
        $data .= $order['number'];
        $data .= $order['summ'];
        $hash = strtoupper(hash('ripemd128', $uid . $guid . md5($data)));
        $guid =
            substr($hash, 0, 8) .
            '-' .
            substr($hash, 8, 4) .
            '-' .
            substr($hash, 12, 4) .
            '-' .
            substr($hash, 16, 4) .
            '-' .
            substr($hash, 20, 12);

        return strtolower($order['cms'] . '-' . $guid);
    }

    function __formatprc($price)
    {
        // форматируем цену согласно настроек
        $price_decimals_point = 2; //ToDo
        $price =
            $price_decimals_point < 0
                ? round($price, $price_decimals_point)
                : number_format($price, $price_decimals_point, '.', '');

        return $price;
    }

    function __checkInn($inn, $type = 1)
    {
        // проверяем заполненность
        if (isset($inn) && trim($inn) != '') {
            if ($type == 2) {
                // только цифры в количестве 10 шт.
                $inn = !preg_match("/^\d{10}$/", trim($inn))
                    ? 'ИНН задан в неверном формате!'
                    : trim($inn);
            } elseif ($type == 1) {
                // только цифры в количестве 12 шт.
                $inn = !preg_match("/^\d{12}$/", trim($inn))
                    ? 'ИНН задан в неверном формате!'
                    : trim($inn);
            } else {
                // только цифры в количестве 10 или 12 шт.
                $inn = !preg_match("/^\d{10}$|^\d{12}$/", trim($inn))
                    ? 'ИНН задан в неверном формате!'
                    : trim($inn);
            }
        } else {
            // ИНН пуст или не найден!
            $inn = 'ИНН пуст или не найден!';
        }

        return $inn;
    }

    function __checkPhone($tel)
    {
        // проверяем заполненность
        if (isset($tel) && trim($tel) != '') {
            // удаляем лишние символы
            $phone = str_replace(
                array('+', '(', ')', '-', ' '),
                '',
                trim($tel)
            );

            // получаем первую цифру
            $first = substr($phone, "0", 1);

            // если номер из 10 цифр и первая цифра не 7, то добавляем 7
            $phone =
                strlen($phone) == 10 && $first != 7 ? '7' . $phone : $phone;

            // окончательная проверка
            $phone = !preg_match("/^[0-9]{11}+$/", $phone)
                ? 'Телефон задан в неверном формате!'
                : $phone;
        } else {
            // Телефон пуст или не найден!
            $phone = 'Телефон пуст или не найден!';
        }

        return $phone;
    }

    function __checkEmail($email)
    {
        // проверка модели записи
        $email = !preg_match("/^\+\d+$|^\S+@\S+$/", $email)
            ? 'E-mail задан в неверном формате!'
            : $email;

        return $email;
    }

    function __checkProductPrice(
        $item_price,
        $order_subtotal,
        $order_discount = 0,
        $order_payment = 0
    ) {
        // если скидка больше нуля
        if ($order_discount > 0) {
            // вычисляем процент скидки
            $percent = ($order_discount * 100) / $order_subtotal;

            // уменьшаем цену товара согласно скидки
            $item_price = $item_price - $item_price * ($percent / 100);
        }

        // если система оплаты больше нуля
        if ($order_payment > 0) {
            // вычисляем процент наценки
            $percent =
                ($order_payment * 100) / ($order_subtotal - $order_discount);

            // увеличиваем цену товара согласно наценки
            $item_price = $item_price + $item_price * ($percent / 100);
        }

        return $item_price;
    }

    function __checkResponce($response)
    {
        $mess = '';

        if (
            is_object($response) &&
            ($response->type == 'ERROR' || $response->status == 400)
        ) {
            $mess = '';
            if ($response->status == 400) {
                $mess .= $response->error . "\n";
                $mess .= $response->message . "\n";
            } else {
                $array = json_decode(json_encode($response), true);
                array_walk_recursive($array, function ($item, $key) use (
                    &$mess
                ) {
                    if ($item != 'ERROR' && trim($item != '')) {
                        $mess .= $key . ': ' . $item . "\n";
                    }
                });
            }
        } elseif (is_object($response)) {
            $array = json_decode(json_encode($response), true);
            array_walk_recursive($array, function ($item, $key) use (&$mess) {
                if (trim($item != '')) {
                    $mess .= $key . ': ' . $item . "\n";
                }
            });
        }

        return $mess;
    }

    function __loadData($url, $headers = false, $data = false, $type = 'POST')
    {
        // инициализируем сеанс cURL
        $ch = curl_init($url);

        // назначаем тип запроса
        if ($type != 'POST') {
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $type);
        } else {
            curl_setopt($ch, CURLOPT_POST, true);
        }

        // добавляем данные
        if ($data) {
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        }

        // возврат результата в качестве строки
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_ENCODING, '');
        curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
        curl_setopt($ch, CURLOPT_TIMEOUT, 0);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);

        // добавляем headers
        if ($headers) {
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        }

        // получаем ответ
        $output = curl_exec($ch);

        // завершаем сеанс cURL
        curl_close($ch);

        return $output;
    }

    function __saveReport(
        $test,
        $bifit,
        $receipt,
        $receipt_id,
        $doc,
        $object,
        $idemp_key
    ) {
        // выводим свойство операции
        $log =
            $test != ''
                ? '####### ' . 'Тестовый запрос' . "\n"
                : '####### ' . 'Боевой запрос' . "\n";

        // устанавливаем часовой пояс согласно настройкам сайта
        date_default_timezone_set('UTC');
        $log .= '####### ' . date('Y-m-d H:i:s', time()) . "\n";

        // ОТВЕТ #1

        $log .= '####### ' . 'Ответ' . ' #1: ' . "\n";

        // данные ответа #1
        $token_error = null;

        if (is_object($bifit)) {
            foreach ($bifit as $key => $val) {
                if ($key == 'error') {
                    $token_error = 1;
                }
                $log .= $key . ': ' . $val . "\n";
            }
        }

        // ЧЕК

        // сформированный чек для отправки
        $log .= '####### ' . 'Чек' . ': ' . "\n" . $receipt . "\n";

        if (!$token_error) {
            // КЛЮЧ ИДЕМПОТЕНТНОСТИ

            $log .=
                '####### ' .
                'Ключ идемпотентности' .
                ': ' .
                "\n" .
                $idemp_key .
                "\n";

            // ОТВЕТ #2

            $log .= '####### ' . 'Ответ' . ' #2: ' . "\n";

            // данные ответа #2
            if (is_object($receipt_id)) {
                $log .= self::__checkResponce($receipt_id);
            } else {
                $log .= 'Номер документа: ' . $receipt_id . "\n";

                // ОТВЕТ #3

                $log .= '####### ' . 'Ответ' . ' #3: ' . "\n";

                if (is_object($doc)) {
                    $log .= self::__checkResponce($doc);
                }

                // ССЫЛКА НА ЧЕК

                // сформированная ссылка на чек
                $log .=
                    '####### ' .
                    'Ссылка на чек' .
                    ': ' .
                    "\n" .
                    $object->receipt_link;
            }
        }

        $object->log = $log;

        // СОХРАНЯЕМ В БД

        $q =
            'INSERT INTO ' .
            _DB_PREFIX_ .
            'bifitkassa_reports (
		order_id,
		order_number,
		order_date,
		action_date,
		receipt_link,
		receipt_id,
		receipt_type,
		log) 
		VALUES (
		' .
            $object->order_id .
            ',
		"' .
            bqSQL($object->order_number) .
            '",
	   "' .
            bqSQL($object->order_date) .
            '",
		"' .
            bqSQL($object->action_date) .
            '",
		"' .
            bqSQL($object->receipt_link) .
            '",
		"' .
            (int) $object->receipt_id .
            '",
		 "' .
            bqSQL($object->receipt_type) .
            '",
		"' .
            bqSQL($object->log) .
            '"
		)';

        Db::getInstance()->Execute($q);
    }

    function __mainProcess($order, $test, $token, $i)
    {
        // 1. АВТОРИЗАЦИЯ
        //    процесс необходимый для подключения клиента к фискальному процессингу

        // адрес для POST-запроса
        $url = 'https://fp' . $test . '.bifit.com/processing-api/oauth/token';

        // данные ссылки
        $data = 'token=' . $token;
        $data .= '&client_id=processing-connector-token';
        $data .= '&client_secret=processing-connector-token';
        $data .= '&grant_type=token';

        // headers
        $headers = array('Content-Type: application/x-www-form-urlencoded');

        // декодируем полученный json-ответ
        $bifit = json_decode(self::__loadData($url, $headers, $data));

        // 2. ФОРМИРОВАНИЕ КОНТЕНТА ЧЕКА
        //    формирование информации для дальнейшей отправки на фискализацию

        // параметры плагина
        /// имя кассира
        $cashier_name = Configuration::get('bifitkassa_cashier_name');
        /// ИНН кассира
        $cashier_inn = Configuration::get('bifitkassa_cashier_inn');
        /// тип документа
        $arr_rec_type = array(
            '1' => 'SALE',
            '2' => 'SALE_RETURN',
            '3' => 'PURCHASE',
            '4' => 'PURCHASE_RETURN'
        );
        $receipt_type = $arr_rec_type[$i];
        /// система налогов
        $tax_system = Configuration::get('bifitkassa_tax_system_' . $i);
        /// признак способа расчёта
        $calc_method = Configuration::get('bifitkassa_calculation_method');
        /// единый НДС
        $vat_mode = Configuration::get('bifitkassa_vat_mode');
        /// НДС
        $vat = Configuration::get('bifitkassa_vat');
        /// тип оплаты
        $payment_type = Configuration::get('bifitkassa_payment_type');
        /// место расчётов
        $payment_address = Configuration::get('bifitkassa_payment_address');
        /// тип адреса клиента
        $address_type = Configuration::get('bifitkassa_address_type');
        /// ФИО клиента
        $fio = Configuration::get('bifitkassa_fio');
        /// адрес клиента в чеке (телефон или эл.почта)
        $order = (object) $order;

        $id_customer = $order->id_customer;
        $customer = new Customer($id_customer);
        $email = $customer->email;

        if ($address_type == 'phone' && $order->phone) {
            $address = self::__checkPhone($order->phone);
        } else {
            $address = self::__checkEmail($email);
        }
        // позиции оплаты заказа
        /// стоимость способа оплаты
        $o_payment = 0;
        /// стоимость упаковки
        $o_package = 0;
        /// стоимость доставки
        $o_shipping = $order->total_shipping_tax_incl;

        // параметры массива
        /// промежуточный итог
        $o_subtotal = 0;
        /// массив товаров для заполнения
        $r_products = array();
        /// массив товаров заказа
        $ProductDetailObject = new OrderDetail();
        $products = $ProductDetailObject->getList($order->id);

        // перебираем массив товаров заказа
        foreach ($products as $key_id => $product) {
            // переводим [array] в [object]
            $product = (object) $product;
            /// корректируем цену товара с учётом НДС
            $prod_price = self::__formatprc($product->unit_price_tax_incl);

            /// корректируем цену товара с учётом скидок
            $p_price = self::__formatprc(
                self::__checkProductPrice(
                    $prod_price,
                    $order->total_products_wt,
                    $order->total_discounts,
                    0
                )
            );
            /// формируем промежуточный итог
            $o_subtotal += $product->total_price_tax_incl;

            /// очищаем имя товара от спец-символов
            $p_name = preg_replace(
                '/[^\p{L}0-9 ]/iu',
                ' ',
                html_entity_decode($product->product_name)
            );
            $p_name = preg_replace('/ +/', ' ', trim($p_name));

            /// формируем массив товаров для чека
            $r_products[$key_id] = new stdClass();
            $r_products[$key_id]->product_name = $p_name;
            $r_products[$key_id]->product_item_price = $p_price;
            $r_products[$key_id]->product_total_price =
                $product->total_price_tax_incl;
            $r_products[$key_id]->product_quantity = $product->product_quantity;
            $r_products[$key_id]->product_tax = 0;
        }

        // сверка итоговых сумм заказа и чека
        $total_1 = self::__formatprc($o_subtotal + $o_package + $o_shipping);
        $total_2 = self::__formatprc($order->total_paid_tax_incl);
        $diff =
            $total_1 > 0 && $total_2 > 0 && $total_1 != $total_2
                ? abs($total_1 - $total_2)
                : 0;

        // параметры для чека
        $items = '';

        // формируем данные чека
        foreach ($r_products as $key_id => $product) {
            /// цена товара
            $p_price = $product->product_item_price;

            /// корректировка цены товара согласно сверки
            if ($diff > 0 && $p_price > $diff) {
                $p_price =
                    $total_2 > $total_1
                        ? $p_price + $diff / $product->product_quantity
                        : $p_price - $diff / $product->product_quantity;
                $diff = 0;
            }

            /// выводим данные товара в чек
            $items .= $items != '' ? ', ' : '';
            $items .= '{';
            $items .= '"calculationMethod":"' . $calc_method . '", ';
            $items .= '"paymentSubject":"PRODUCT", ';
            $items .= '"name":"' . $product->product_name . '", ';
            $items .= '"price":' . self::__formatprc($p_price) . ', ';
            $items .= '"quantity":' . $product->product_quantity . ', ';
            $items .=
                $vat_mode == 1
                    ? '"vat":"' . $vat . '", '
                    : '"vat":"VAT_' . floor($product->product_tax) . '", ';
            $items .=
                '"total":' . self::__formatprc($product->product_total_price);
            $items .= '}';
        }

        $pactax = 0;
        // упаковка
        if ($o_package > 0) {
            // выводим данные упаковки в чек
            $items .= $items != '' ? ', ' : '';
            $items .= '{';
            $items .= '"calculationMethod":"' . $calc_method . '", ';
            $items .= '"paymentSubject":"PRODUCT", ';
            $items .= '"name":"Упаковка", ';
            $items .= '"price":' . self::__formatprc($o_package) . ', ';
            $items .= '"quantity":1, ';
            $items .=
                $vat_mode == 1
                    ? '"vat":"' . $vat . '", '
                    : '"vat":"VAT_' . floor($pactax) . '", ';
            $items .= '"total":' . self::__formatprc($o_package);
            $items .= '}';
        }

        // доставка
        if ($o_shipping > 0) {
            /// выводим данные способа доставки в чек
            $items .= $items != '' ? ', ' : '';
            $items .= '{';
            $items .= '"calculationMethod":"' . $calc_method . '", ';
            $items .= '"paymentSubject":"SERVICE", ';
            $items .= '"name":"Доставка", ';
            $items .= '"price":' . self::__formatprc($o_shipping) . ', ';
            $items .= '"quantity":1, ';
            $items .=
                $vat_mode == 1
                    ? '"vat":"' . $vat . '", '
                    : '"vat":"VAT_' . floor($shptax) . '", ';
            $items .= '"total":' . self::__formatprc($o_shipping);
            $items .= '}';
        }

        $id_customer = $order->id_customer;

        $customer = new Customer($id_customer);

        // формируем чек
        $receipt = '{';
        $receipt .=
            '"type":"' .
            $receipt_type .
            '", "taxSystem":"' .
            $tax_system .
            '", ';
        $receipt .= '"cashier":{';
        $receipt .= '"name":"' . $cashier_name . '", ';
        $receipt .= '"inn":"' . self::__checkInn($cashier_inn, 1) . '"';
        $receipt .= '}, ';
        $receipt .= '"client":{';
        $receipt .= $fio
            ? '"name":"' .
                trim($customer->firstname . ' ' . $customer->lastname) .
                '", '
            : '';
        $receipt .= '"address":"' . $address . '"';
        $receipt .= '}, ';
        $receipt .= '"paymentAddress":"' . $payment_address . '", ';
        $receipt .= '"items":[' . $items . '], ';
        $receipt .=
            '"total":' . self::__formatprc($order->total_paid_tax_incl) . ', ';
        $receipt .=
            '"payments":{"' .
            $payment_type .
            '": ' .
            self::__formatprc($order->total_paid_tax_incl) .
            '}';
        $receipt .= '}';

        // 3. ОТПРАВКА ЧЕКА В ФИСКАЛЬНЫЙ ПРОЦЕССИНГ
        //    передача контента чека в фискальный процессинг для дальнейшей фискализации

        // адрес для POST-запроса
        $url =
            'https://fp' .
            $test .
            '.bifit.com/processing-api/protected/documents/registration/receipts';

        // данные чека
        $data = $receipt;

        // ключ идемпотентности (Idempotency-Key)
        $o = [];
        $o['number'] = $order->id;
        $o['summ'] = self::__formatprc($order->total_paid_tax_incl);
        $o['cms'] = 'PrestaShop';
        $idemp_key = self::__createGUID($o);

        // headers
        $headers = array(
            'Content-Type: application/json',
            'Authorization: Bearer ' . $bifit->access_token,
            'Idempotency-Key: ' . $idemp_key
        );

        // получаем идентификатор документа на процессинге (id)
        $receipt_id = json_decode(self::__loadData($url, $headers, $data));

        // 4. ПОЛУЧЕНИЕ ДОКУМЕНТА ПО ID
        //    получение документа с фискальными признаками по его номеру

        // параметры
        $doc = '';

        if (!is_object($receipt_id)) {
            // ссылка для GET-запроса
            $url =
                'https://fp' .
                $test .
                '.bifit.com/processing-api/protected/documents/' .
                $receipt_id;

            // headers
            $headers = array('Authorization: Bearer ' . $bifit->access_token);

            // получаем документ
            $doc = json_decode(self::__loadData($url, $headers, false, 'GET'));
        }

        // ССЫЛКА НА ЧЕК

        // формируем ссылку на чек
        $receipt_link = !is_object($receipt_id)
            ? 'https://fp' .
                $test .
                '.bifit.com/processing-api/receipts/' .
                $idemp_key
            : 'XXX';

        // ИСТОРИЯ ОПЕРАЦИИ

        // параметры объекта
        $object = new stdClass();
        /// id заказа
        $object->order_id = $order->id;
        /// номер заказа
        $object->order_number = $order->id;
        /// дата заказа
        $object->order_date = $order->date_add;
        /// дата операции
        $object->action_date = date('Y-m-d H:i:s');
        /// ссылка на чек
        $object->receipt_link = $receipt_link;
        /// id полученного документа
        $object->receipt_id = !is_object($receipt_id) ? $receipt_id : '000';
        /// тип проводимого документа
        $object->receipt_type = $receipt_type;
        /// лог операции (будет заполнен далее)
        $object->log = '';

        // сохраняем операцию в историю
        self::__saveReport(
            $test,
            $bifit,
            $receipt,
            $receipt_id,
            $doc,
            $object,
            $idemp_key
        );
    }

    public function hookActionOrderStatusUpdate($params)
    {
        $order = new Order($params["id_order"]);

        // данные заказа
        /// статус заказа
        $status_id = $params["newOrderStatus"]->id;
        /// способ оплаты
        $payment_id = 0;
        $paymentMethod = $order->module;
        $modules_list = Module::getPaymentModules();
        foreach ($modules_list as $module) {
            if ($module["name"] == $paymentMethod) {
                $payment_id = $module['id_module'];
            }
        }

        // параметры плагина
        /// выбранный режим работы
        $test = Configuration::get('bifitkassa_test') ? '-test' : '';
        /// токен для соединения
        $token =
            $test != ''
                ? 'P5cKbUUD9uSSrSlGdzspLblvBnD0GzTAE0cLmAPSEMxJ79DtLE'
                : Configuration::get('token');

        // перебираем вкладки типов документа
        for ($i = 1; $i <= 4; $i++) {
            // узнаём нужно ли использовать данный тип документа
            $use_type = (int) Configuration::get('bifitkassa_use_type_' . $i);
            // если использовать нужно
            if ($use_type) {
                // получаем параметры
                /// выбранный способ(ы) оплаты
                $pm_ids = json_decode(
                    Configuration::get('bifitkassa_pm_ids_' . $i),
                    true
                );
                /// выбранный статус(ы) заказа
                $pm_statuses = json_decode(
                    Configuration::get('bifitkassa_pm_statuses_' . $i),
                    true
                );

                // Если совпадает способ оплаты и статус заказа
                if (
                    isset($pm_ids) &&
                    isset($pm_statuses) &&
                    in_array($payment_id, $pm_ids) &&
                    in_array($status_id, $pm_statuses) &&
                    $token != ''
                ) {
                    // запускаем БИФИТ-процесс
                    self::__mainProcess($order, $test, $token, $i);
                }
            }
        }
    }

    function SampleFunction($arg)
    {
        global $cookie;

        return (int) $cookie->id_employee;
    }
}