<?php

/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2019 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class BK extends ObjectModel
{
	/** @var string Name */
	public $content;
	public $title;
	public $position;
	public $active;
	public $info;

	/**
	 * @see ObjectModel::$definition
	 */
	public static $definition = array(
		'table' => 'c_reports',
		'primary' => 'id_bifitkassa_reports',
		'multilang' => true,
		'fields' => array(
			'position' => array('type' => self::TYPE_INT),
			'active' => array('type' => self::TYPE_BOOL),
			'info' => array('type' => self::TYPE_STRING),
			// Lang fields
			'title' => array(
				'type' => self::TYPE_STRING,
				'lang' => true,
				'validate' => 'isGenericName',
				'required' => true,
				'size' => 128
			),
			'content' => array(
				'type' => self::TYPE_HTML,
				'lang' => true,
				'validate' => 'isString',
				'size' => 3999999999999
			)
		)
	);

	public function add($autodate = true, $null_values = false)
	{
		$this->position = self::getLastPosition();

		return parent::add($autodate, true);
	}

	public function update($null_values = false)
	{
		if (parent::update($null_values)) {
			return $this->cleanPositions();
		}

		return false;
	}

	public function delete()
	{
		die('Test1!');
		if (parent::delete()) {
			return $this->cleanPositions();
		}

		return false;
	}

	public static function getLastPosition()
	{
		$sql =
			'
		SELECT MAX(position) + 1
		FROM `' .
			_DB_PREFIX_ .
			'bifitkassa_reports`';

		return Db::getInstance()->getValue($sql);
	}

	public static function cleanPositions()
	{
		$sql =
			'
		SELECT `id_bifitkassa_reports`
		FROM `' .
			_DB_PREFIX_ .
			'bifitkassa_reports`
		ORDER BY `position`';

		$result = Db::getInstance()->executeS($sql);

		for ($i = 0, $total = count($result); $i < $total; ++$i) {
			$sql =
				'UPDATE `' .
				_DB_PREFIX_ .
				'bifitkassa_reports`
					SET `position` = ' .
				(int) $i .
				'
					WHERE `id_bifitkassa_reports` = ' .
				(int) $result[$i]['id_bifitkassa_reports'];
			Db::getInstance()->execute($sql);
		}

		return true;
	}

	public function updatePosition($way, $position)
	{
		if (
			!($res = Db::getInstance()->executeS(
				'
			SELECT cp.`id_bifitkassa_reports`, cp.`position`
			FROM `' .
					_DB_PREFIX_ .
					'bifitkassa_reports` cp
			ORDER BY cp.`position` ASC'
			))
		) {
			return false;
		}

		foreach ($res as $row) {
			if ((int) $row['id_bifitkassa_reports'] == (int) $this->id) {
				$moved_row = $row;
			}
		}

		if (!isset($moved_row) || !isset($position)) {
			return false;
		}

		// < and > statements rather than BETWEEN operator
		// since BETWEEN is treated differently according to databases
		return Db::getInstance()->execute(
			'
			UPDATE `' .
				_DB_PREFIX_ .
				'bifitkassa_reports`
			SET `position`= `position` ' .
				($way ? '- 1' : '+ 1') .
				'
			WHERE `position`
			' .
				($way
					? '> ' .
						(int) $moved_row['position'] .
						' AND `position` <= ' .
						(int) $position
					: '< ' .
						(int) $moved_row['position'] .
						' AND `position` >= ' .
						(int) $position)
		) &&
			Db::getInstance()->execute(
				'
			UPDATE `' .
					_DB_PREFIX_ .
					'bifitkassa_reports`
			SET `position` = ' .
					(int) $position .
					'
			WHERE `id_bifitkassa_reports` = ' .
					(int) $moved_row['id_bifitkassa_reports']
			);
	}
}